@@ -20,11 +20,13 @@ from importlib import metadata
 from importlib.machinery import ModuleSpec, SourceFileLoader
 from importlib.metadata import PackageNotFoundError
 from itertools import chain
+from logging import getLogger
 from pathlib import Path, PureWindowsPath, WindowsPath
 from pprint import pformat
 from subprocess import run
 from types import ModuleType
-from typing import Any, Callable, Optional, Union
+from typing import Any, Callable, Optional, Protocol, TypeVar, Union, runtime_checkable
+from warnings import catch_warnings, filterwarnings, warn
 
 import furl
 import packaging
@@ -33,15 +35,23 @@ from furl import furl as URL
 from icontract import ensure, require
 from packaging import tags
 from packaging.specifiers import SpecifierSet
-from pip._internal.utils import compatibility_tags
+
 
 import use
-from use import AmbiguityWarning, Hash, Modes, config
+from use import Hash, Modes, VersionWarning, config
 from use.hash_alphabet import JACK_as_num, hexdigest_as_JACK, num_as_hexdigest
 from use.messages import Message, _web_no_version_or_hash_provided
 from use.pypi_model import PyPI_Project, PyPI_Release, Version, _delete_none
 from use.tools import pipes
 
+log = getLogger(__name__)
+
+
+T = TypeVar("T", bound=Callable[[str, str, Version, set[str]], str])
+@runtime_checkable
+class MissingHashFotmatter(Protocol[T]):
+    pass
+
 
 class PlatformTag:
     def __init__(self, platform: str):
@@ -68,7 +78,7 @@ def _ensure_version(
         return result
     result_version = _get_version(mod=result)
     if result_version != version:
-        return AmbiguityWarning(Message.version_warning(name, version, result_version))
+        warn(Message.version_warning(name, version, result_version), category=VersionWarning)
     return result
 
 
@@ -80,9 +90,9 @@ def _ensure_path(value: Union[bytes, str, furl.Path, Path]) -> Path:
     if isinstance(value, furl.Path):
         return (
             Path.cwd(),
-            value.segments 
-            << map(Path) 
-            << tuple 
+            value.segments
+            << map(Path)
+            << tuple
             << reduce(Path.__truediv__),
         ) << reduce(Path.__truediv__)
     return value
@@ -100,7 +110,7 @@ def execute_wrapped(sql: str, params: tuple):
 
 
 @cache
-def get_supported() -> frozenset[PlatformTag]:
+def get_supported() -> frozenset[PlatformTag]: # cov: exclude
     """
     Results of this function are cached. They are expensive to
     compute, thanks to some heavyweight usual players
@@ -110,10 +120,35 @@ def get_supported() -> frozenset[PlatformTag]:
     Returns a set containing all platform _platform_tags
     supported on the current system.
     """
-    items: list[PlatformTag] = []
+    get_supported = None
+    with catch_warnings():
+        filterwarnings(action="ignore", category=DeprecationWarning)
+        try:
+          from pip._internal.resolution.legacy.resolver import get_supported
+        except ImportError:
+          pass
+        if not get_supported:
+          try:
+            from pip._internal.models.target_python import get_supported
+          except ImportError:
+            pass
+        if not get_supported:
+          try:
+            from pip._internal.utils.compatibility_tags import get_supported
+          except ImportError:
+            pass
+        if not get_supported:
+          try:
+            from pip._internal.resolution.resolvelib.factory import get_supported
+          except ImportError:
+            pass
+    
+    get_supported = get_supported or (lambda: [])
+
+    items: list[PlatformTag] = [
+        PlatformTag(platform=tag.platform) for tag in get_supported()
+    ]
 
-    for tag in compatibility_tags.get_supported():
-        items.append(PlatformTag(platform=tag.platform))
     for tag in packaging.tags._platform_tags():
         items.append(PlatformTag(platform=str(tag)))
 
@@ -121,18 +156,17 @@ def get_supported() -> frozenset[PlatformTag]:
 
 
 def _filter_by_version(project_: "PyPI_Project", version: str) -> "PyPI_Project":
+    v = Version(version)
+    rels = project_.releases.get(v, project_.releases.get(version, []))
 
-    for_version = (
-        project_.releases.get(version)
-        or project_.releases.get(str(version))
-        or project_.releases.get(Version(str(version)))
-    )
-    new_data = {
-        "urls": for_version,
-        "releases": {Version(str(version)): for_version},
-        "info": project_.info.dict(),
-    }
-    return PyPI_Project(**new_data)
+    if not rels:
+      return project_
+
+    project_.releases = { v: rels }
+    project_.urls = rels
+    print(repr(project_))
+
+    return project_
 
 
 class ZipFunctions:
@@ -239,7 +273,7 @@ def _pebkac_no_version(
     hash_algo=None,
     package_name: str = None,
     module_name: str = None,
-    message_formatter: Callable[[str, str, Version, set[str]], str] = Message.pebkac_missing_hash,
+    message_formatter: MissingHashFotmatter = Message.pebkac_missing_hash,
     **kwargs,
 ) -> Union[ModuleType, Exception]:
 
@@ -258,15 +292,13 @@ def _pebkac_no_hash(
     package_name: str = None,
     **kwargs,
 ) -> Union[Exception, ModuleType]:
-    try:
-        hashes = {
-            hexdigest_as_JACK(entry.digests.get(hash_algo.name))
-            for entry in (_get_package_data(package_name).releases[version])
-        }
-        if not hashes:
-            rw = RuntimeWarning(Message.pebkac_unsupported(package_name))
-        return rw
-    except (IndexError, KeyError) as ike:
+    hashes = {
+        hexdigest_as_JACK(entry.digests.get(hash_algo.name))
+        for entry in _get_package_data(package_name).releases[version]
+    }
+    if hashes:
+        return RuntimeWarning(Message.pebkac_missing_hash(name=package_name, package_name=package_name, version=version, hashes=hashes))
+    else:
         return RuntimeWarning(Message.no_distribution_found(package_name, version))
 
 
@@ -277,17 +309,17 @@ def _pebkac_no_version_no_hash(
     hash_algo: Hash,
     package_name: str,
     **kwargs,
-) -> Union[Exception, ModuleType]:
+) -> Exception:
     # let's try to make an educated guess and give a useful suggestion
     proj = _get_package_data(package_name)
     ordered = _filtered_and_ordered_data(proj, version=None)
     # we tried our best, but we didn't find anything that could work'
     if not ordered:
         return RuntimeWarning(Message.pebkac_unsupported(package_name))
-
     # we found something that could work, but it may not fit to the user's requirements
-    hashes = {o.digests.get(hash_algo.name) for o in proj.urls}
     version = ordered[0].version
+    hash = ordered[0].digests.get(hash_algo.name)
+    hashes = { hexdigest_as_JACK(hash) }
     return RuntimeWarning(
         Message.no_version_or_hash_provided(
             name=name,
@@ -390,7 +422,7 @@ def _auto_install(
     mod = None
     if "installation_path" not in query or not _ensure_path(query["installation_path"]).exists():
         if query:
-            use.del_entry(name, version)
+            sys.modules["use"].del_entry(name, version)
         query = _find_or_install(package_name, version, force_install=True)
         artifact_path = _ensure_path(query["artifact_path"])
         module_path = _ensure_path(query["module_path"])
@@ -420,7 +452,7 @@ def _auto_install(
         if "fault_inject" in config:
             config["fault_inject"](**locals())
         if mod:
-            use.main._save_module_info(
+            sys.modules["use"]._save_module_info(
                 name=package_name,
                 import_relpath=str(_ensure_path(module_path).relative_to(installation_path)),
                 version=version,
@@ -442,9 +474,9 @@ def _process(*argv, env={}):
                 args=[*map(str, argv)],
                 bufsize=1024,
                 input="",
-                capture_output=True,
+                capture_output=False,
                 timeout=45000,
-                check=False,
+                check=True,
                 close_fds=True,
                 env=_realenv,
                 encoding="ISO-8859-1",
@@ -629,6 +661,8 @@ def _find_or_install(name, version=None, artifact_path=None, url=None, out_info=
             "pip",
             "--disable-pip-version-check",
             "--no-color",
+            "--verbose",
+            "--verbose",
             "install",
             "--pre",
             "--root",
@@ -648,8 +682,6 @@ def _find_or_install(name, version=None, artifact_path=None, url=None, out_info=
             "--no-warn-conflicts",
             install_item,
         )
-        sys.stderr.write("\n\n".join((output.stderr, output.stdout)))
-
     site_dir, module_path = _find_module_in_venv(package_name, version, relp)
     module_paths = []
     if module_path:
@@ -681,27 +713,25 @@ def _load_venv_entry(*, package_name, module_name, installation_path, module_pat
     with open(module_path, "rb") as code_file:
         try:
             for variant in (
+                cwd,
                 installation_path,
                 Path(str(str(installation_path).replace("lib64/", "lib/"))),
                 Path(str(str(installation_path).replace("lib/", "lib64/"))),
-                None,
+                installation_path,
             ):
-                if not variant:
-                    raise RuntimeError()
                 if not variant.exists():
                     continue
+                origcwd = Path.cwd()
                 try:
-                    os.chdir(cwd)
+                    log.info("Changimg directory: from %s to %s", origcwd, variant)
                     os.chdir(variant)
-                    return _build_mod(
-                        name=(module_name.replace("/", ".")),
-                        code=code_file.read(),
-                        module_path=_ensure_path(module_path),
-                        initial_globals={},
-                    )
+                    return importlib.import_module(module_name.replace("/", "."))
                 except ImportError as ierr0:
                     orig_exc = orig_exc or ierr0
                     continue
+                finally:
+                    log.debug("Change directory back: from %s to %s", Path.cwd(), origcwd)
+                    os.chdir(origcwd)
         except RuntimeError as ierr:
             try:
                 return importlib.import_module(module_name)
@@ -736,11 +766,25 @@ def _filter_by_platform(
                 rel,
                 sys_version=sys_version,
                 platform_tags=tags,
-                include_sdist=True,
+                include_sdist=False,
             )
         ]
         for ver, releases in project.releases.items()
     }
+    if not filtered:
+      filtered = {
+          ver: [
+              rel.dict()
+              for rel in releases
+              if _is_compatible(
+                  rel,
+                  sys_version=sys_version,
+                  platform_tags=tags,
+                  include_sdist=True,
+              )
+          ]
+          for ver, releases in project.releases.items()
+      }
 
     return PyPI_Project(**{**project.dict(), **{"releases": filtered}})
 
@@ -752,17 +796,21 @@ def _filtered_and_ordered_data(data: PyPI_Project, version: Version = None) -> l
         filtered = (
             data
             >> _filter_by_version(version)
-            # >> _filter_by_platform(tags=get_supported(), sys_version=_sys_version())  # let's not filter by platform for now
+            >> _filter_by_platform(tags=get_supported(), sys_version=sys.version_info[0:2])  # let's not filter by platform for now
         )
     else:
         filtered = data
-        # filtered = _filter_by_platform(data, tags=get_supported(), sys_version=_sys_version())
+        filtered = _filter_by_platform(data, tags=get_supported(), sys_version=sys.version_info[0:2])
 
     flat = reduce(list.__add__, filtered.releases.values(), [])
     return sorted(
         flat,
-        key=lambda r: (r.filename.endswith(".tar.gz"), r.is_sdist, r.version),
-        reverse=False,
+        key=(lambda r: (
+            1 - int(r.filename.endswith(".tar.gz")),
+            1 - int(r.is_sdist),
+            r.version,
+        )),
+        reverse=True,
     )
 
 
@@ -784,20 +832,17 @@ def _is_platform_compatible(
     info: PyPI_Release, platform_tags: frozenset[PlatformTag], include_sdist=False
 ) -> bool:
 
-    if "py2" in info.justuse.python_tag and "py3" not in info.justuse.python_tag:
-        return False
-
     if not include_sdist and (".tar" in info.justuse.ext or info.justuse.python_tag in ("cpsource", "sdist")):
         return False
 
-    if "win" in info.packagetype and sys.platform != "win32":
-        return False
-
-    if "win32" in info.justuse.platform_tag and sys.platform != "win32":
-        return False
-
-    if "macosx" in info.justuse.platform_tag and sys.platform != "darwin":
+    if "win" in (info.packagetype or "unknown") and sys.platform != "win32":
         return False
+    
+    if info.platform_tag:
+        if "win32" in info.platform_tag and sys.platform != "win32":
+            return False
+        if "macosx" in info.platform_tag and sys.platform != "darwin":
+            return False
 
     our_python_tag = tags.interpreter_name() + tags.interpreter_version()
     supported_tags = set(
@@ -810,16 +855,20 @@ def _is_platform_compatible(
         ]
     )
 
-    given_platform_tags = info.justuse.platform_tag.split(".") << map(PlatformTag) >> frozenset
-
-    if info.is_sdist and info.requires_python is not None:
+    if info.platform_tag:
+        given_platform_tags = info.platform_tag.split(".") << map(PlatformTag) >> frozenset
+    else:
+        return include_sdist
+    
+    if info.is_sdist and info.requires_python:
         given_python_tag = {
             our_python_tag
             for p in info.requires_python.split(",")
             if Version(platform.python_version()) in SpecifierSet(p)
         }
     else:
-        given_python_tag = set(info.justuse.python_tag.split("."))
+        given_python_tag = set(info.python_tag.split(".")) 
+    
     return any(supported_tags.intersection(given_python_tag)) and (
         (info.is_sdist and include_sdist) or any(given_platform_tags.intersection(platform_tags))
     )
@@ -841,15 +890,21 @@ def _apply_aspect(
     thing,
     check,
     pattern,
-    decorator: Callable[[Callable[..., Any]], Any],
+    decorator,
     aspectize_dunders=False,
 ) -> Any:
     """Apply the aspect as a side-effect, no copy is created."""
     for name, obj in thing.__dict__.items():
+        if isinstance(obj, (dict, list, set, tuple, str, bytes)):
+            continue
+        if name == "beartype": continue
+        module = getattr(obj, "__module__", None)
+        if module in ("builtins", "collections", "collections.abc", "functools", "inspect", "itertools", "logging", "pprint", "subprocess", "textwrap", "typing", "warnings",): continue
         if not aspectize_dunders and name.startswith("__") and name.endswith("__"):
             continue
         if check(obj) and re.match(pattern, name):
             thing.__dict__[name] = decorator(obj)
+            log.debug(f"Applied {decorator.__name__} to {module}::{obj.__qualname__} [{obj.__class__.__qualname__}]")
     return thing
 
 
@@ -916,4 +971,4 @@ def _fail_or_default(exception: BaseException, default: Any):
     if default is not Modes.fastfail:
         return default  # TODO: write test for default
     else:
-        raise exception
+        raise exception
\ No newline at end of file
